/********************************************************
 Name          : gpio_int_pwm_adc.c
 Author        : adp@unipi.gr
 Copyright     : 
 Description   : ---LED3  , 
 		 ---LED1    ,
 		 ---LED5     ,
 		 ---LED6     
 **********************************************************/

#include "compiler.h"
#include "intc.h"
#include "gpio.h"
#include "pwm.h"
#include "adc.h"
#include "board.h"

#define PWM_PER 128
#define PWM_DUR 127
unsigned int channel_id;


//         ADC
//      LED1,LED5  LED6
#if __GNUC__
	__attribute__((__interrupt__))
#elif __ICCAVR32__
__interrupt
#endif

static void ADC_int_handler(void)
{		
volatile avr32_pwm_t *pwm = &AVR32_PWM;
volatile avr32_adc_t *adc = &AVR32_ADC;
int LED6Rint,LED5Rint,LED1int;
int tempr,light,pot;

//  ADC
tempr=adc_get_value(adc,0);
pot=adc_get_value(adc,1);
light=adc_get_value(adc,2);
//    
LED6Rint=PWM_DUR*tempr/1024;
pwm->channel[2].cupd=LED6Rint;
LED5Rint=PWM_DUR*light/1024;
pwm->channel[0].cupd=LED5Rint;
LED1int=PWM_DUR*pot/1024;
pwm->channel[4].cupd=LED1int;
}

//  	
int main(void)
{
pwm_opt_t pwm_opt;        
avr32_pwm_channel_t pwm_channel;		
extern unsigned int channel_id;
	 
Disable_global_interrupt();
INTC_init_interrupts();
		
//   2  PWM   GPIO pin 53 (PB21-LED6 )
gpio_enable_module_pin(AVR32_PWM_2_PIN, AVR32_PWM_2_FUNCTION);
//   0  PWM   GPIO pin 51 (PB19-LED5 )
gpio_enable_module_pin(AVR32_PWM_0_PIN, AVR32_PWM_0_FUNCTION);
//   4  PWM   GPIO pin 59 (PB27-LED1)
gpio_enable_module_pin(AVR32_PWM_4_1_PIN, AVR32_PWM_4_1_FUNCTION);

//   0  ADC   GPIO pin 21 (PA21-)
gpio_enable_module_pin(AVR32_ADC_AD_0_PIN, AVR32_ADC_AD_0_FUNCTION);
//   1  ADC   GPIO pin 22 (PA22-)
gpio_enable_module_pin(AVR32_ADC_AD_1_PIN, AVR32_ADC_AD_1_FUNCTION);
//   2  ADC   GPIO pin 23 (PA23-)
gpio_enable_module_pin(AVR32_ADC_AD_2_PIN, AVR32_ADC_AD_2_FUNCTION);

//  PWM MR
//  PWM      
pwm_opt.diva = AVR32_PWM_DIVA_CLK_OFF;
pwm_opt.divb = AVR32_PWM_DIVB_CLK_OFF;
pwm_opt.prea = AVR32_PWM_PREA_MCK;
pwm_opt.preb = AVR32_PWM_PREB_MCK;
pwm_init(&pwm_opt);
	  
//   PWM 2 ( LED6 - )
channel_id = 2;
pwm_channel.CMR.calg = PWM_MODE_LEFT_ALIGNED;       //  
pwm_channel.CMR.cpol = PWM_POLARITY_LOW;            //  
pwm_channel.CMR.cpd = PWM_UPDATE_DUTY;              //  duty cycle
pwm_channel.CMR.cpre = AVR32_PWM_CPRE_MCK_DIV_4;    //   115200/4 Hz
pwm_channel.cdty = PWM_DUR;  
pwm_channel.cprd = PWM_PER; 
pwm_channel.cupd = 0;   //  UPD    CDTY
pwm_channel_init(channel_id, &pwm_channel); //   2
pwm_start_channels(1 << channel_id);  //   2

//   PWM 0 ( LED5 - )
channel_id = 0;
pwm_channel.CMR.calg = PWM_MODE_LEFT_ALIGNED;       //  
pwm_channel.CMR.cpol = PWM_POLARITY_LOW;            //  
pwm_channel.CMR.cpd = PWM_UPDATE_DUTY;              //  duty cycle
pwm_channel.CMR.cpre = AVR32_PWM_CPRE_MCK_DIV_4;    //   115200/4 Hz
pwm_channel.cdty = PWM_DUR;
pwm_channel.cprd = PWM_PER;  
pwm_channel.cupd = 0;   //  UPD    CDTY
pwm_channel_init(channel_id, &pwm_channel); //   0
pwm_start_channels(1 << channel_id);  //   0

//   PWM 4 (LED1 - )
channel_id = 4;
pwm_channel.CMR.calg = PWM_MODE_LEFT_ALIGNED;       //  
pwm_channel.CMR.cpol = PWM_POLARITY_LOW;            //  
pwm_channel.CMR.cpd = PWM_UPDATE_DUTY;              //  duty cycle
pwm_channel.CMR.cpre = AVR32_PWM_CPRE_MCK_DIV_4;    //   115200/4 Hz
pwm_channel.cdty = PWM_DUR; 
pwm_channel.cprd = PWM_PER;
pwm_channel.cupd = 0;   //  UPD    CDTY
pwm_channel_init(channel_id, &pwm_channel); //   0
pwm_start_channels(1 << channel_id);  //   0

//  ADC
volatile avr32_adc_t *adc = &AVR32_ADC;
// set Sample/Hold time to max so that the ADC capacitor should be loaded entirely
adc->mr |= 0xF << AVR32_ADC_SHTIM_OFFSET;
// set Startup to max so that the ADC capacitor should be loaded entirely
adc->mr |= 0x1F << AVR32_ADC_STARTUP_OFFSET;
//  ADC = MCK/64
adc->mr |= 0x3F << AVR32_ADC_PRESCAL_OFFSET;
//      
adc->ier |= 1 << AVR32_ADC_IER_EOC2_OFFSET;
//   0,1,2
adc->cher = 7; //(1+2+4)

//  
INTC_register_interrupt(&ADC_int_handler,AVR32_ADC_IRQ, AVR32_INTC_INT0);     
Enable_global_interrupt();

int i;
while (1)				//  
	{
	for (i = 0; i < 500; i += 1) 
    	adc_start(adc);			//   
	gpio_tgl_gpio_pin(LED2_GPIO);	//  LED3
	}
}
